--	Copyright (C) 2004 by neuron										  
--	aagaande@gmail.com											 
--																		  
--	This program is free software; you can redistribute it and--or modify  
--	it under the terms of the GNU General Public License as published by  
--	the Free Software Foundation; either version 2 of the License, or	 
--	(at your option) any later version.								   
--																		  
--	This program is distributed in the hope that it will be useful,	   
--	but WITHOUT ANY WARRANTY; without even the implied warranty of		
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the		 
--	GNU General Public License for more details.						  
--																		  
--	You should have received a copy of the GNU General Public License	 
--	along with this program; if not, write to the						 
--	Free Software Foundation, Inc.,									   
--	59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 

local moduleName = "SmartVisibility"
local SmartVisibility = nQuestLog:NewModule(moduleName,"AceEvent-2.0","AceHook-2.1", "AceConsole-2.0")
local quixote = AceLibrary("Quixote-1.0")

local L = AceLibrary("AceLocale-2.2"):new(moduleName)
L:RegisterTranslations("enUS", function() return {
	["Description"] = "Deals with showing/hiding things automatically",

	["Zone"] = true,
	["Auto fold zones"] = true,
	["your not in"] = true,
	["Auto unfold zones"] = true,
	["your enter"] = true,
	
	["Objectives"] = true,
	["Hide old quest objectives"] = true,
	["Hide quest objectives older than X minutes"] = true,
	["set to 0 to disable"] = true,
	["Note : This will only work if quests objectives -> hide all objectives is on"] = true,
	["Show objectives for new quests"] = true,
	
	["showCurrentZone"] = "Show objectives for current zone",
	["showCurrentZoneLong"] = "Always show quest objectives for current zone",
} end)

function SmartVisibility:OnModuleLoad()
	-- We'd want to set self.name and self.description here? (used for the menu made by parent atleast)
	self.description = L["Description"]
	
	self.core:RegisterDefaults(moduleName, "profile", {
		["enabled"] = true,
		
		["zoneAutoFold"] = true,
		["zoneAutoUnfold"] = true,

		["hideOldObjectives"] = 30, -- This translates to hiding quest objectives older than 30 minutes, 0 = disable.
		["newOnGained"] = true, -- Count gained quests as new quest
		["showCurrentZone"] = false,
	})
end

function SmartVisibility:OnZoneChange()
	local currentZone = GetZoneText()
	if (currentZone == "") then
		return
	end
	self.currentZone = currentZone
	
	for _, zone in quixote:IterZones() do
		if self.db.profile.zoneAutoFold and zone ~= currentZone then
			-- Auto fold zone we're not in
			self.core.db.char.hiddenZones[zone] = true
		elseif self.db.profile.zoneAutoUnfold and zone == currentZone then
			-- Auto unfold zone we are in
			self.core.db.char.hiddenZones[zone] = nil
		end
	end
	self.core:Refresh() -- TODO, find a way for modules to mark needed refresh, and then do one refresh instead of many.
end

function SmartVisibility:ShouldShowObjective(title)
	if (self.db.profile.showCurrentZone) then
		local _, _, _, _, _, _, zone = quixote:GetQuestByName(title)
		if (zone == self.currentZone) then
			return true
		end
	end


	if (self.db.profile.hideOldObjectives > 0) then
		local minTime = GetTime() - (self.db.profile.hideOldObjectives * 60)
		if self.questTimes[title] and self.questTimes[title] > minTime then
			-- It's new, return true
			return true
		end
		-- When we have no data, we assume the quest is old
		return false
	end
	
	
	return true
end

function SmartVisibility:OnQuest_Gained(title)
	if self.db.profile.newOnGained then
		self.questTimes[title] = GetTime()
	end
end

function SmartVisibility:OnQuest_Lost(title)
	self.questTimes[title] = nil
end

function SmartVisibility:OnQuest_Leaderboard_Update(title, _, _, desc)
	self.questTimes[title] = GetTime()
end

function SmartVisibility:ToggleEnabled(v)
	self.questTimes = {} -- stores last update to quest
	
	if (v) then
		self:RegisterEvent("Quixote_Quest_Gained", "OnQuest_Gained")
		self:RegisterEvent("Quixote_Quest_Lost", "OnQuest_Lost")
		self:RegisterEvent("Quixote_Leaderboard_Update", "OnQuest_Leaderboard_Update")
		
		self:RegisterEvent("ZONE_CHANGED", "OnZoneChange")
		self:RegisterEvent("ZONE_CHANGED_INDOORS", "OnZoneChange")
		self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "OnZoneChange")
		self:RegisterEvent("Quixote_Ready", "OnZoneChange",true)
	else
		self:UnregisterEvent("Quixote_Quest_Gained", "OnQuest_Gained")
		self:UnregisterEvent("Quixote_Quest_Lost", "OnQuest_Lost")
		self:UnregisterEvent("Quixote_Leaderboard_Update", "OnQuest_Leaderboard_Update")
		
		self:UnregisterEvent("ZONE_CHANGED", "OnZoneChange")
		self:UnregisterEvent("ZONE_CHANGED_INDOORS", "OnZoneChange")
		self:UnregisterEvent("ZONE_CHANGED_NEW_AREA", "OnZoneChange")
	end
end

function SmartVisibility:GetMenu(oldMenu)
	local menu = {
		type="group",
		name=moduleName,
		order=oldMenu.order,
		desc=self.description,
		args = {
			toggleEnabled = oldMenu,
			
			zoneHeader = {
				order = oldMenu.order + 1,
				type="header",
				name=L["Zone"],
				description=L["Zone"],
			},
			autoFold = {
				order = oldMenu.order + 2,
				type = "toggle",
				name = L["Auto fold zones"],
				desc = L["Auto fold zones"] .. " " .. L["your not in"],
				get = function()
					return self.db.profile.zoneAutoFold
				end,
				set = function(v)
					self.db.profile.zoneAutoFold = v
					self.core:OnZoneChange()
				end,
			},
			autoUnfold = {
				order = oldMenu.order + 3,
				type = "toggle",
				name = L["Auto unfold zones"],
				desc = L["Auto fold zones"] .. " " .. L["your enter"],
				get = function()
					return self.db.profile.zoneAutoUnfold
				end,
				set = function(v)
					self.db.profile.zoneAutoUnfold = v
					self.core:OnZoneChange()
				end,
			},
			
			objectivesHeader = {
				order = oldMenu.order + 4,
				type="header",
				name=L["Objectives"],
				description=L["Objectives"],
			},
			
			currentZone = {
				order = oldMenu.order + 7,
				type = "toggle",
				name = L["showCurrentZone"],
				desc = L["showCurrentZoneLong"],
				get = function()
					return self.db.profile.showCurrentZone
				end,
				set = function(v)
					self.db.profile.showCurrentZone = v
					self.core:Refresh()
				end,
			},
			hideOld = {
				order = oldMenu.order + 5,
				type = "range",
				name = L["Hide old quest objectives"],
				desc = L["Hide quest objectives older than X minutes"] .. " (" .. L["set to 0 to disable"] .. ")" .. "\n" .. L["Note : This will only work if quests objectives -> hide all objectives is on"],
				get = function()
					return self.db.profile.hideOldObjectives
				end,
				set = function(v)
					self.db.profile.hideOldObjectives = v
					self.core:Refresh()
				end,
				min=0,
				max=120,
				step=5,
			},
			newOnGained = {
				order = oldMenu.order + 6,
				type = "toggle",
				name = " - " .. L["Show objectives for new quests"],
				desc = L["Show objectives for new quests"],
				get = function()
					return self.db.profile.newOnGained
				end,
				set = function(v)
					self.db.profile.newOnGained = v
					self.core:Refresh()
				end,
			},

			
		}
	}
	return menu
end
